function countIngredients(ingredientList)
	local numOfIngredients = 0
	for _,oneIngredient in pairs(ingredientList) do
		if oneIngredient[2] == nil then
			numOfIngredients = numOfIngredients + oneIngredient.amount
		else
			numOfIngredients = numOfIngredients + oneIngredient[2]
		end
	end
	return numOfIngredients
end
	
function craftTime(ingredientCount, techLvl)
	return math.ceil(string.format("%.2f", techLvl + math.log(ingredientCount^(techLvl + 1) + 1)) * 4) / 4
end

--*** 'fix' recipes

data.raw.recipe["big-electric-pole"].ingredients = {
	{
		"steel-plate",
		5
	},
	{
		"copper-cable",
		data.raw["electric-pole"]["big-electric-pole"].maximum_wire_distance / 2 --so one wire reaches 2 tiles! is that enough?
	}
}

data.raw.recipe["medium-electric-pole"].ingredients = {
	{
		"steel-plate",
		2
	},
	{
		"copper-cable",
		data.raw["electric-pole"]["medium-electric-pole"].maximum_wire_distance / 2
	}
}

data.raw.recipe.substation.ingredients = {
	{
		"steel-plate",
		10
	},
	{
		"advanced-circuit",
		5
	},
	{
		"copper-cable",
		data.raw["electric-pole"].substation.maximum_wire_distance / 2
	}
}

data.raw.recipe["red-wire"].ingredients = {
	{
		"copper-cable",
		2
	},
	{
		"plastic-bar",
		1
	}
}

data.raw.recipe["green-wire"].ingredients = {
	{
		"copper-cable",
		2
	},
	{
		"plastic-bar",
		1
	}
}

data.raw.recipe["engine-unit"].ingredients = {
	{
		"steel-plate",
		1
	},
	{
		"iron-gear-wheel",
		1
	},
	{
		"pipe",
		2
	},
	{
		amount = 5,
		name = "lubricant",
		type = "fluid"
	}
}

data.raw.recipe["electric-engine-unit"].ingredients = {
	{
		"iron-plate",
		2
	},
	{
		"steel-plate",
		5
	},
	{
		"copper-cable",
		15
	},
	{
		"advanced-circuit",
		5
	}
}

data.raw.recipe["rail-signal"].ingredients = {
	{
		"electronic-circuit",
		1
	},
	{
		"iron-plate",
		5
	},
	{
		"small-lamp",
		3
	}
}

data.raw.recipe["rail-chain-signal"].ingredients = {
	{
		"rail-signal",
		1
	},
	{
		"electronic-circuit",
		2
	}
}

data.raw.recipe.concrete.ingredients = {
	{
		"stone",
		10
	},
	{
		"iron-stick",
		4
	},
	{
		"iron-ore",
		1
	},
	{
		amount = 100,
		name = "water",
		type = "fluid"
	}
}

data.raw.recipe["steel-chest"].ingredients = {
	{
		"steel-plate",
		7
	},
	{
		"iron-chest",
		1
	}
}

data.raw.recipe["steel-furnace"].ingredients = {
	{
		"steel-plate",
		6
	},
	{
		"stone-brick",
		8
	},
	{
		"stone-furnace",
		1
	}
}

data.raw.recipe["splitter"].ingredients = {
	{
		"electronic-circuit",
		2
	},
	{
		"iron-plate",
		2
	},
	{
		"transport-belt",
		2
	},
	{
		"iron-gear-wheel",
		2
	}
}

data.raw.recipe["submachine-gun"].expensive.ingredients = {
	{
		"copper-plate",
		20
	},
	{
		"iron-plate",
		20
	}
	-- {
		-- "iron-spring",
		-- 10
	-- }
}

data.raw.recipe["submachine-gun"].normal.ingredients = {
	{
		"copper-plate",
		5
	},
	{
		"iron-plate",
		10
	}
	-- {
		-- "iron-spring",
		-- 5
	-- }
}

data.raw.recipe["gun-turret"].ingredients = {
	{
		"submachine-gun",
		1
	},
	{
		"iron-gear-wheel",
		5
	},
	{
		"iron-plate",
		10
	},
	{
		"steel-plate",
		2
	}
}

--*** adjust a few things

data.raw["assembling-machine"]["assembling-machine-1"]["crafting_speed"] = 0.75
data.raw["assembling-machine"]["assembling-machine-2"]["crafting_speed"] = 1.0

--*** add new items

local steamLoco = table.deepcopy(data.raw["locomotive"]["locomotive"])
steamLoco.name = "steam-locomotive"
steamLoco.minable.result = "steam-locomotive"
steamLoco.breaking_force = 5
steamLoco.max_health = 500
steamLoco["max-power"] = "300kW"
steamLoco["max-speed"] = 0.8
steamLoco.working_sound.sound.filename = "__base__/sound/steam-engine-90bpm.ogg"
data:extend{steamLoco}
data.raw.recipe["steam-locomotive"] = {
	enabled = false,
	energy_required = 4,
	ingredients = {
		{
			"steam-engine",
			1
		},
		{
			"pipe",
			10
		},
		{
			"steel-plate",
			20
		}
	},
	name = "steam-locomotive",
	result = "steam-locomotive",
	type = "recipe"
}
data.raw["item-with-entity-data"]["steam-locomotive"] = {
	flags = {
		"goes-to-quickbar"
	},
	icon = "__base__/graphics/icons/diesel-locomotive.png",
	icon_size = 32,
	name = "steam-locomotive",
	order = "a[train-system]-f[diesel-locomotive]",
	place_result = "steam-locomotive",
	stack_size = 5,
	subgroup = "transport",
	type = "item-with-entity-data"
}
--tech

data.raw["ammo-turret"]["armored-gun-turret"] = {
	alert_when_attacking = true,
	attack_parameters = {
		ammo_category = "bullet",
		cooldown = 6,
		projectile_center = {
			0,
			-0.0875
		},
		projectile_creation_distance = 1.39375,
		range = 18,
		shell_particle = {
			center = {
				-0.0625,
				0
			},
			creation_distance = -1.925,
			direction_deviation = 0.1,
			name = "shell-particle",
			speed = 0.1,
			speed_deviation = 0.03,
			starting_frame_speed = 0.2,
			starting_frame_speed_deviation = 0.1
		},
		sound = {
			{
				filename = "__base__/sound/fight/heavy-gunshot-1.ogg",
				volume = 0.45
			},
			{
				filename = "__base__/sound/fight/heavy-gunshot-2.ogg",
				volume = 0.45
			},
			{
				filename = "__base__/sound/fight/heavy-gunshot-3.ogg",
				volume = 0.45
			},
			{
				filename = "__base__/sound/fight/heavy-gunshot-4.ogg",
				volume = 0.45
			}
		},
		type = "projectile"
	},
	attacking_animation = {
		layers = {
			{
				axially_symmetrical = false,
				direction_count = 64,
				frame_count = 2,
				height = 64,
				shift = {
					0,
					-0.875
				},
				stripes = {
					{
						filename = "__base__/graphics/entity/gun-turret/gun-turret-gun-1.png",
						height_in_frames = 32,
						width_in_frames = 2
					},
					{
						filename = "__base__/graphics/entity/gun-turret/gun-turret-gun-2.png",
						height_in_frames = 32,
						width_in_frames = 2
					}
				},
				width = 66
			},
			{
				apply_runtime_tint = true,
				axially_symmetrical = false,
				direction_count = 64,
				filename = "__base__/graphics/entity/gun-turret/gun-turret-gun-mask.png",
				flags = {
					"mask"
				},
				frame_count = 2,
				height = 27,
				line_length = 2,
				shift = {
					0.015625,
					-1.01563
				},
				width = 29
			},
			{
				axially_symmetrical = false,
				direction_count = 64,
				draw_as_shadow = true,
				frame_count = 2,
				height = 50,
				shift = {
					1.23438,
					0
				},
				stripes = {
					{
						filename = "__base__/graphics/entity/gun-turret/gun-turret-gun-shadow-1.png",
						height_in_frames = 32,
						width_in_frames = 2
					},
					{
						filename = "__base__/graphics/entity/gun-turret/gun-turret-gun-shadow-2.png",
						height_in_frames = 32,
						width_in_frames = 2
					}
				},
				width = 91
			}
		}
	},
	attacking_speed = 0.5,
	automated_ammo_count = 10,
	base_picture = {
		layers = {
			{
				axially_symmetrical = false,
				direction_count = 1,
				filename = "__base__/graphics/entity/gun-turret/gun-turret-base.png",
				frame_count = 1,
				height = 75,
				priority = "high",
				shift = {
					0,
					-0.046875
				},
				width = 90
			},
			{
				apply_runtime_tint = true,
				axially_symmetrical = false,
				direction_count = 1,
				filename = "__base__/graphics/entity/gun-turret/gun-turret-base-mask.png",
				flags = {
					"mask"
				},
				frame_count = 1,
				height = 47,
				line_length = 1,
				shift = {
					0,
					-0.234375
				},
				width = 52
			}
		}
	},
	call_for_help_radius = 40,
	collision_box = {
		{
			-0.7,
			-0.7
		},
		{
			0.7,
			0.7
		}
	},
	corpse = "big-remnants",
	dying_explosion = "big-explosion",
	flags = {
		"placeable-player",
		"player-creation"
	},
	folded_animation = {
		layers = {
			{
				axially_symmetrical = false,
				direction_count = 4,
				filename = "__base__/graphics/entity/gun-turret/gun-turret-gun-extension.png",
				frame_count = 1,
				height = 63,
				line_length = 1,
				priority = "medium",
				run_mode = "forward",
				shift = {
					0.015625,
					-0.859375
				},
				width = 65
			},
			{
				apply_runtime_tint = true,
				axially_symmetrical = false,
				direction_count = 4,
				filename = "__base__/graphics/entity/gun-turret/gun-turret-gun-extension-mask.png",
				flags = {
					"mask"
				},
				frame_count = 1,
				height = 31,
				line_length = 1,
				run_mode = "forward",
				shift = {
					0,
					-0.890625
				},
				width = 24
			},
			{
				axially_symmetrical = false,
				direction_count = 4,
				draw_as_shadow = true,
				filename = "__base__/graphics/entity/gun-turret/gun-turret-gun-extension-shadow.png",
				frame_count = 1,
				height = 49,
				line_length = 1,
				run_mode = "forward",
				shift = {
					1.20313,
					0.015625
				},
				width = 89
			}
		}
	},
	folding_animation = {
		layers = {
			{
				axially_symmetrical = false,
				direction_count = 4,
				filename = "__base__/graphics/entity/gun-turret/gun-turret-gun-extension.png",
				frame_count = 5,
				height = 63,
				line_length = 0,
				priority = "medium",
				run_mode = "backward",
				shift = {
					0.015625,
					-0.859375
				},
				width = 65
			},
			{
				apply_runtime_tint = true,
				axially_symmetrical = false,
				direction_count = 4,
				filename = "__base__/graphics/entity/gun-turret/gun-turret-gun-extension-mask.png",
				flags = {
					"mask"
				},
				frame_count = 5,
				height = 31,
				line_length = 0,
				run_mode = "backward",
				shift = {
					0,
					-0.890625
				},
				width = 24
			},
			{
				axially_symmetrical = false,
				direction_count = 4,
				draw_as_shadow = true,
				filename = "__base__/graphics/entity/gun-turret/gun-turret-gun-extension-shadow.png",
				frame_count = 5,
				height = 49,
				line_length = 0,
				run_mode = "backward",
				shift = {
					1.20313,
					0.015625
				},
				width = 89
			}
		}
	},
	folding_speed = 0.08,
	icon = "__base__/graphics/icons/gun-turret.png",
	icon_size = 32,
	inventory_size = 1,
	max_health = 800,
	minable = {
		mining_time = 0.5,
		result = "armored-gun-turret"
	},
	name = "armored-gun-turret",
	prepared_animation = {
		layers = {
			{
				axially_symmetrical = false,
				direction_count = 64,
				frame_count = 1,
				height = 64,
				shift = {
					0,
					-0.875
				},
				stripes = {
					{
						filename = "__base__/graphics/entity/gun-turret/gun-turret-gun-1.png",
						height_in_frames = 32,
						width_in_frames = 1
					},
					{
						filename = "__base__/graphics/entity/gun-turret/gun-turret-gun-2.png",
						height_in_frames = 32,
						width_in_frames = 1
					}
				},
				width = 66
			},
			{
				apply_runtime_tint = true,
				axially_symmetrical = false,
				direction_count = 64,
				filename = "__base__/graphics/entity/gun-turret/gun-turret-gun-mask.png",
				flags = {
					"mask"
				},
				frame_count = 1,
				height = 27,
				line_length = 1,
				shift = {
					0.015625,
					-1.01563
				},
				width = 29
			},
			{
				axially_symmetrical = false,
				direction_count = 64,
				draw_as_shadow = true,
				frame_count = 1,
				height = 50,
				shift = {
					1.23438,
					0
				},
				stripes = {
					{
						filename = "__base__/graphics/entity/gun-turret/gun-turret-gun-shadow-1.png",
						height_in_frames = 32,
						width_in_frames = 1
					},
					{
						filename = "__base__/graphics/entity/gun-turret/gun-turret-gun-shadow-2.png",
						height_in_frames = 32,
						width_in_frames = 1
					}
				},
				width = 91
			}
		}
	},
	preparing_animation = {
		layers = {
			{
				axially_symmetrical = false,
				direction_count = 4,
				filename = "__base__/graphics/entity/gun-turret/gun-turret-gun-extension.png",
				frame_count = 5,
				height = 63,
				line_length = 0,
				priority = "medium",
				run_mode = "forward",
				shift = {
					0.015625,
					-0.859375
				},
				width = 65
			},
			{
				apply_runtime_tint = true,
				axially_symmetrical = false,
				direction_count = 4,
				filename = "__base__/graphics/entity/gun-turret/gun-turret-gun-extension-mask.png",
				flags = {
					"mask"
				},
				frame_count = 5,
				height = 31,
				line_length = 0,
				run_mode = "forward",
				shift = {
					0,
					-0.890625
				},
				width = 24
			},
			{
				axially_symmetrical = false,
				direction_count = 4,
				draw_as_shadow = true,
				filename = "__base__/graphics/entity/gun-turret/gun-turret-gun-extension-shadow.png",
				frame_count = 5,
				height = 49,
				line_length = 0,
				run_mode = "forward",
				shift = {
					1.20313,
					0.015625
				},
				width = 89
			}
		}
	},
	preparing_speed = 0.08,
	rotation_speed = 0.015,
	selection_box = {
		{
			-1,
			-1
		},
		{
			1,
			1
		}
	},
	type = "ammo-turret",
	vehicle_impact_sound = {
		filename = "__base__/sound/car-metal-impact.ogg",
		volume = 0.65
	}
}
data.raw.recipe["armored-gun-turret"] = {
	enabled = false,
	energy_required = 8,
	ingredients = {
		{
			"gun-turret",
			1
		},
		{
			"steel-plate",
			20
		},
		--[[
		{
			"iron-gear-wheel",
			10
		},
		{
			"copper-plate",
			10
		},
		{
			"iron-plate",
			20
		}
		--]]
	},
	name = "armored-gun-turret",
	result = "armored-gun-turret",
	type = "recipe"
}
data.raw.item["armored-gun-turret"] = {
	flags = {
		"goes-to-quickbar"
	},
	icon = "__base__/graphics/icons/gun-turret.png",
	icon_size = 32,
	name = "armored-gun-turret",
	order = "b[turret]-a[gun-turret]",
	place_result = "armored-gun-turret",
	stack_size = 50,
	subgroup = "defensive-structure",
	type = "item"
}
--tech for armored-gun-turret



--*** remove stupid items (could 'fix' them too)

data.raw["electric-pole"]["small-electric-pole"]["supply_area_distance"] = 3.5
data.raw["electric-pole"]["small-electric-pole"]["max_health"] = 75
-- data.raw.recipe["small-electric-pole"] = nil
-- data.raw.item["small-electric-pole"] = nil

-- 'fix' crafting times
local itemTechlvl = {}
for _,tech in pairs(data.raw.technology) do
	if tech.effects then
		for _,effect in pairs(tech.effects) do
			if effect.type == "unlock-recipe" then
				itemTechlvl[effect.recipe] = #tech.unit.ingredients
			end
		end
	end
end

for _,recipe in pairs(data.raw.recipe) do
	local lvl
	if itemTechlvl[recipe.name] then
		lvl = itemTechlvl[recipe.name] + 1
	else
		lvl = 1
	end
	--log("")
	if recipe.expensive == nil then
		--local gameTime = recipe.energy_required or 0.5
		--log(recipe.name.." game craft Time ".. gameTime)
		
		recipe.energy_required = craftTime(countIngredients(recipe.ingredients), lvl)
		
		--log("new craft time "..recipe.energy_required)
	else
		--local gameTimeE = recipe.expensive.energy_required or 0.5
		--local gameTimeN = recipe.normal.energy_required or 0.5
		--log(recipe.name.." game craft Time expensive ".. gameTimeE)
		--log(recipe.name.." game craft Time normal ".. gameTimeN)
		
		recipe.expensive.energy_required = craftTime(countIngredients(recipe.expensive.ingredients), lvl)
		
		--log("new craft time expensive "..recipe.expensive.energy_required)
		
		recipe.normal.energy_required = craftTime(countIngredients(recipe.normal.ingredients), lvl)
		
		--log("new craft time normal "..recipe.normal.energy_required)
	end
end